#!/bin/sh

if echo -n | egrep -e -n >/dev/null ; then
    dashn=""
else
    dashn=-n
fi

print_interp()
{
	if [ -f /bin/sun ] && /bin/sun ; then
	        set `uname -a`
	        ch=`echo $3 | cut -c1`
	        case $ch in
	        4)      echo sunos4 ;;
	        5)      echo solaris2 ;;
	        *)      echo UNKNOWN ;;
	        esac
	elif [ `uname` = "AIX" ] ; then
	        case `uname -vr` in
	        "2 3") echo aix3-r2 ;;
	        "1 4") echo aix4-r1 ;;
	        "2 4") echo aix4-r1 ;;
	        "3 4") echo aix4-r1 ;;
	        *) exit 1 ;;
	        esac
	else
		if [ `uname` = "Linux" ] ; then
			echo linux-ix86
		else
			uname
		fi
	fi
}

check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

check_path()
{
	if [ -d $1 ] ; then
		return 0
	fi

	return 1
}

make_path()
{
	check_path $1
	if [ $? != 0 ] ; then
		mkdir -p $1
	fi
	check_path $1
	return $?
}

auto_start()
{
proxy=epproxy
INTERP=${interp}
ETRC=$install_dir/$proxy.sh
CRCF="/etc/rc3.d/S999$proxy \
      /etc/rc0.d/K49$proxy \
      /etc/rc1.d/K49$proxy \
      /etc/rc2.d/K49$proxy"
DEVNULL=/dev/null
DATE=`date`
# add /usr/sbin to path for AIX to find mkitab
PATH=$PATH:/usr/sbin

    
case "$INTERP" in
    
linux-ix86|linux-s390)
    # SuSE looks for kill scripts in prev level. All other look in new runlevel
    if [ -f /etc/SuSE-release ]; then 
      CRCF="/etc/rc.d/rc2.d/S999$proxy \
            /etc/rc.d/rc3.d/S999$proxy \
            /etc/rc.d/rc4.d/S999$proxy \
            /etc/rc.d/rc5.d/S999$proxy \
            /etc/rc.d/rc2.d/K9$proxy \
            /etc/rc.d/rc3.d/K9$proxy \
            /etc/rc.d/rc4.d/K9$proxy \
            /etc/rc.d/rc5.d/K9$proxy"
    else
      CRCF="/etc/rc.d/rc2.d/S999$proxy \
            /etc/rc.d/rc3.d/S999$proxy \
            /etc/rc.d/rc4.d/S999$proxy \
            /etc/rc.d/rc5.d/S999$proxy \
            /etc/rc.d/rc0.d/K9$proxy \
            /etc/rc.d/rc1.d/K9$proxy \
            /etc/rc.d/rc6.d/K9$proxy"
    fi
    ;;
esac

#echo "RC files are: $CRCF"

#################
# Setup autostart
################# 
case "$INTERP" in
	aix3-r2|aix4-r1)
		RCFILE=/etc/rc.$proxy
		RCTAB=$proxy
		INITTAB=/etc/inittab
		INITSAVE=$INITTAB.before.$proxy

		if [ ! -f $RCFILE ]
		then
			set +e
			cat >> $RCFILE <<EOF
#!/bin/sh
#
#		Start the Tivoli $proxy
#
if [ -f $ETRC ]; then
    $ETRC start
fi
EOF
			# If the creation of the /etc/rc.$proxy file fails, then
			# we need to exit, cleaning up any changes we might have
			# made.
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			chmod 0774 $RCFILE
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			if lsitab rcnfs >/dev/null 2>&1
			then
				RCENTRY=rcnfs
			else
				RCENTRY=rctcpip
			fi

			#
			#  Okay - we've successfully added the rc startup file,
			# need to modify /etc/inittab
			#
			cp $INITTAB $INITSAVE
			mkitab -i $RCENTRY "$RCTAB:2:wait:$RCFILE > /dev/console 2>&1 # Tivoli $proxy"
			if [ $? -ne 0 ]
				then
				rmitab $RCTAB
				rm -rf $RCFILE
				cp $INITSAVE $INITTAB
				rm -rf $INITSAVE
				echo "Unable to modify /etc/inittab, removing $proxy installation"
				return 1
			fi
			set -e
		fi
	    ;;
	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    if [ "$INTERP" = "linux-ix86" -o "$INTERP" = "linux-s390" ]; then
	        MRCF="$MRCD/$proxy"
		if [ -d /var/lock/subsys ]; then
			touch /var/lock/subsys/$proxy
		fi
	    else
	        MRCF="$MRCD/$proxy.rc"
	    fi

	    if [ ! -d $MRCD ]; then
	        MRCF="$ETRC"
	    fi
	    if test -f $MRCF && cmp -s $MRCF $ETRC >$DEVNULL 2>&1; then
	        echo "$MRCF Tivoli $proxy daemon autostart file is already present."
	    else
	        rm -f $MRCF
	    	cp -p $ETRC $MRCF
		echo "Tivoli $proxy daemon master autostart file is $MRCF."
	    fi
	    for i in $CRCF; do
		if [ -f $i ]; then
		    rm -f $i;
		fi
		# the convention for hpux10 and sgi is to always
		# symlink startup scripts from init.d/script to
		# the corresponding rc?.d/script
		case "$INTERP" in
		    hpux10|mips-irix5|linux-ix86|linux-s390)
			ln -s $MRCF $i
			;;
		    *)
	        	ln $MRCF $i >$DEVNULL 2>&1
			if [ $? -ne 0 ]; then
			    ln -s $MRCF $i
			fi
			;;
		esac
	    done
            ;;
    esac

    return 0
}


work_dir=`pwd`

interp=`print_interp`

if [ X${interp} != Xaix4-r1 ] && [ X${interp} != Xsolaris2 ] && [ X${interp} != Xlinux-ix86 ] ; then
	echo "Platform $interp is not supported."
	exit 1
fi


ok=0
while [ $ok = 0 ] ; do
	echo $dashn "To install the Tivoli Endpoint Proxy, you must accept the agreement written in the License file."
	echo $dashn "If you accept the agreement, enter Y.  [Y/N]: "
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
		ok=1
	fi

	if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
         [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
		exit 1
	fi
done

ok=0
default=/usr/proxy
while [ $ok = 0 ] ; do
	echo $dashn "Installation directory [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
	else
		try_path=${default}
	fi

	check_path ${try_path}
	if [ $? != 0 ] ; then
		echo $dashn "$try_path does not exist. Create it? [Y/N]: "
		check_response
		if [ $? = 0 ] ; then
			make_path $try_path
		fi
	fi

	check_path ${try_path}
	if [ $? = 0 ] ; then
		install_dir=${try_path}
		ok=1
	fi
done

ok=0
default=nobody
while [ $ok = 0 ] ; do
	echo $dashn "User account which proxy should be run as [default=$default]: "
	read ans
	if [ X${ans} = X ] ; then
		username=$default
	else	
		username=$ans
	fi
	ok=1
done

# set permissions on installed files
chown $username $install_dir
chmod 750 $install_dir
cp epproxy wproxy libcpl* libdes* libmrt* $install_dir/
chown $username $install_dir/*
chmod 700 $install_dir/*
mkdir $install_dir/C
cp epp.cat $install_dir/C


ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Gateway address: "
	read ans
	if [ X${ans} != X ] ; then
		gateway_host=$ans
		ok=1
	else
		ok=0
	fi
done

ok=0
default=9494
while [ $ok = 0 ] ; do
	echo $dashn "Gateway port [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			gateway_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			gateway_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			gateway_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			gateway_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		gateway_port=$default
		ok=1
	fi
done

gateway_address="${gateway_host}+${gateway_port}"

ok=0
while [ $ok = 0 ] ; do
	echo $dashn "Endpoint proxy port: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			children_local_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		ok=0
	fi
done


done=0
while [ $done = 0 ] ; do
	ok=0
	while [ $ok = 0 ] ; do
		echo $dashn "Relay or gateway proxy hostname: "
		read ans
		if [ X${ans} != X ] ; then
			peer_host=$ans
			ok=1
		else
			ok=0
		fi
	done

	ok=0
	while [ $ok = 0 ] ; do
		echo $dashn "Relay or gateway proxy port: "
		read ans
		if [ X${ans} != X ] ; then
			case $ans in
			[1-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			[1-9][0-9][0-9][0-9][0-9])
				peer_port=$ans
				ok=1
				;;
			*)
				echo "Invalid port number -> " $ans
				;;
			esac
		else
			ok=0
		fi
	done

	if [ X${peers} != X ] ; then
		peers="${peers};${peer_host}+${peer_port}"
	else
		peers="${peer_host}+${peer_port}"
	fi

	default=N
	echo $dashn "Enter more destinations? [Y/N] [default=$default]: "
	check_response
	if [ $? != 0 ] ; then
		done=1
	fi
done

#
# Connection manager type
#
ok=0
connection_mode=client
while [ $ok = 0 ] ; do
	echo $dashn "Specify how endpoint proxy connects to destination [0=bidirectional, 1=unidirectional initiator, 2=unidirectional listener]: "
	read ans
    if [ X${ans} = X0 ] ; then
        connection_manager_type=cm-tcp-bidirectional
        ok=1
    elif [ X${ans} = X1 ] ; then
        connection_manager_type=cm-tcp-unidirectional
        connection_mode=client
        ok=1
    elif [ X${ans} = X2 ] ; then
        connection_manager_type=cm-tcp-unidirectional
        connection_mode=server
        ok=1
    else
    	echo "Invalid choice -> " $ans
	fi
done

logfile_path="epp.log"

nfile=unlimited
ulimit_solaris2="ulimit -n $nfile"
ulimit_linux="ulimit -n 32768"

if [ X${interp} = Xsolaris2 ] ; then
	ulimit_cmd=${ulimit_solaris2}
fi

if [ X${interp} = Xlinux-ix86 ] ; then
	ulimit_cmd=${ulimit_linux}
fi

# create the configuration file for the proxy
cat >$install_dir/epproxy.cfg <<EOF

[endpoint-proxy]
 
# The address and port the TME Gateway is listening on
gateway-host = $gateway_address

# The network interface to use for listening for Endpoint traffic (from TME Gateway)
#gateway-interface

# Timeout interval (in seconds) for TCP/IP operations to succeed/fail
#tcpip-timeout

# Timeout interval (in seconds) the proxy waits for expected connections from the TME Gateway
#accept-timeout

# List of port ranges to use when allocating endpoint ports (e.g. n1|n1-m1[,n2|n2-m2 ... ])
#port-range

# The maximum number of connections that the proxy will service at a time
#max-sessions

# Full path to a directory where the Endpoint Proxy database should be located
#database-path

# Disable use of UDP (for initial logins) between Endpoint Proxy and TME Gateway
#disable-udp


[log]

# Sets the level of debug [0-11] (default = 0)
debug-level = 3

# Full path to the log file where the Relay messages are written (default = <STDERR>)
log-file = $logfile_path

# The max size of the log file in MegaBytes (if 0 the log file size is unlimited) (default = 1)
max-size = 0


[communication-layer]
# The network interface to use for listening traffic from children (Relay/Gateway Proxy(s))
#children-local-host = 

# The port to listen on for traffic from children (Relay/Gateway Proxy(s))
children-local-port = $children_local_port

# List of children hosts (e.g. hostname1+port1[;hostname2+port2 ...])
children-remote-list = $peers

# The communication interface used to connect to children
children-cm-type = $connection_manager_type

[children-cm-info]
# Connection mode (unidirectional only, may be "client" or "server")
connection-mode = $connection_mode
EOF

# create the script to set environment
cat >$install_dir/setup_env.sh <<EOF
#!/bin/sh
# AIX
LIBPATH=${install_dir}:$LIBPATH
export LIBPATH
# Solaris, Linux
LD_LIBRARY_PATH=${install_dir}:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH
EOF


# create the script to run the proxy
cat >$install_dir/epproxy.sh <<EOF
#!/bin/sh
# AIX
LIBPATH=${install_dir}:$LIBPATH
export LIBPATH
# Solaris, Linux
LD_LIBRARY_PATH=${install_dir}:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH

NLSPATH=$install_dir/%L/%N.cat:$install_dir/%l/%N.cat:$install_dir/C/%N.cat\${NLSPATH:+":\$NLSPATH"}
export NLSPATH

${ulimit_cmd}

[ -d "$install_dir" ] && cd $install_dir

case "\$1" in
	
	stop_msg)
		shift
		echo "Stopping the epproxy daemon"
		;;
	stop)
		shift
		if [ -f $install_dir/epproxy.pid ] ; then
			pid=\`sed -n 1p $install_dir/epproxy.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep epproxy | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						rm -f \$install_dir/epproxy.pid
						kill -15 \$pid
						exit \$?
					fi
				done
				echo "The process id in $install_dir/epproxy.pid is not an epproxy process id."
				echo "Unable to get process id of the epproxy daemon (pid)."
			else
				echo "Unable to get process id of the epproxy daemon (pid)."
				exit 1
			fi
		else
			echo "Unable to get process id of the epproxy daemon (epproxy.pid)."
			exit 1
		fi
		;;
	start_msg)
		shift
		echo "Starting the epproxy daemon"
		;;
	start|*)
		if [ "\$1" = "start" ] ; then
			shift
		fi

		if [ -f $install_dir/epproxy.pid ] ; then
			pid=\`sed -n 1p $install_dir/epproxy.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep epproxy | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						echo "epproxy for this environment already running under pid \$pid"
						exit 1
					fi
				done
			fi
		fi

		su - $username -c "NLSPATH=\$NLSPATH LIBPATH=\$LIBPATH LD_LIBRARY_PATH=\$LD_LIBRARY_PATH $install_dir/epproxy $install_dir" &

		;;
esac
EOF

# set permissions on the script
chown $username $install_dir/epproxy.sh
chmod 500 $install_dir/epproxy.sh

auto_start

ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the endpoint proxy? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/epproxy.sh start 1>/dev/null 2>&1 &
	else
		echo "Endpoint proxy not started"
	fi
	ok=1
done
