#!/bin/sh

if echo -n | egrep -e -n >/dev/null ; then
	dashn=""
else
	dashn=-n
fi

print_interp()
{
	if [ -f /bin/sun ] && /bin/sun ; then
	        set `uname -a`
	        ch=`echo $3 | cut -c1`
	        case $ch in
	        4)      echo sunos4 ;;
	        5)      echo solaris2 ;;
	        *)      echo UNKNOWN ;;
	        esac
	elif [ `uname` = "AIX" ] ; then
	        case `uname -vr` in
	        "2 3") echo aix3-r2 ;;
	        "1 4") echo aix4-r1 ;;
	        "2 4") echo aix4-r1 ;;
	        "3 4") echo aix4-r1 ;;
	        *) exit 1 ;;
	        esac
	else
		if [ `uname` = "Linux" ] ; then
			echo linux-ix86
		else
			uname
		fi
	fi
}

check_response()
{
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ||
         [ "$1" = "Y" -a X${ans} = X ] ; then
		return 0
	fi

	return 1
}

check_path()
{
	if [ -d $1 ] ; then
		return 0
	fi

	return 1
}

make_path()
{
	check_path $1
	if [ $? != 0 ] ; then
		mkdir -p $1
	fi
	check_path $1
	return $?
}

auto_start()
{
proxy=eventsink
INTERP=${interp}
ETRC=$install_dir/$proxy.sh
CRCF="/etc/rc3.d/S99$proxy \
      /etc/rc0.d/K49$proxy \
      /etc/rc1.d/K49$proxy \
      /etc/rc2.d/K49$proxy"
DEVNULL=/dev/null
DATE=`date`
# add /usr/sbin to path for AIX to find mkitab
PATH=$PATH:/usr/sbin

    
case "$INTERP" in
    
linux-ix86|linux-s390)
    # SuSE looks for kill scripts in prev level. All other look in new runlevel
    if [ -f /etc/SuSE-release ]; then 
      CRCF="/etc/rc.d/rc2.d/S99$proxy \
            /etc/rc.d/rc3.d/S99$proxy \
            /etc/rc.d/rc4.d/S99$proxy \
            /etc/rc.d/rc5.d/S99$proxy \
            /etc/rc.d/rc2.d/K9$proxy \
            /etc/rc.d/rc3.d/K9$proxy \
            /etc/rc.d/rc4.d/K9$proxy \
            /etc/rc.d/rc5.d/K9$proxy"
    else
      CRCF="/etc/rc.d/rc2.d/S99$proxy \
            /etc/rc.d/rc3.d/S99$proxy \
            /etc/rc.d/rc4.d/S99$proxy \
            /etc/rc.d/rc5.d/S99$proxy \
            /etc/rc.d/rc0.d/K9$proxy \
            /etc/rc.d/rc1.d/K9$proxy \
            /etc/rc.d/rc6.d/K9$proxy"
    fi
    ;;
esac

#echo "RC files are: $CRCF"

#################
# Setup autostart
################# 
case "$INTERP" in
	aix3-r2|aix4-r1)
		RCFILE=/etc/rc.$proxy
		RCTAB=$proxy
		INITTAB=/etc/inittab
		INITSAVE=$INITTAB.before.$proxy

		if [ ! -f $RCFILE ]
		then
			set +e
			cat >> $RCFILE <<EOF
#!/bin/sh
#
#		Start the Tivoli $proxy
#
if [ -f $ETRC ]; then
    $ETRC start
fi
EOF
			# If the creation of the /etc/rc.$proxy file fails, then
			# we need to exit, cleaning up any changes we might have
			# made.
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			chmod 0774 $RCFILE
			if [ $? -ne 0 ]
			then
				rm -rf $RCFILE
				echo "Unable to install Tivoli $proxy"
				return 1
			fi

			if lsitab rcnfs >/dev/null 2>&1
			then
				RCENTRY=rcnfs
			else
				RCENTRY=rctcpip
			fi

			#
			#  Okay - we've successfully added the rc startup file,
			# need to modify /etc/inittab
			#
			cp $INITTAB $INITSAVE
			mkitab "$RCTAB:2:wait:$RCFILE > /dev/console 2>&1 # Tivoli $proxy"
			if [ $? -ne 0 ]
				then
				rmitab $RCTAB
				rm -rf $RCFILE
				cp $INITSAVE $INITTAB
				rm -rf $INITSAVE
				echo "Unable to modify /etc/inittab, removing $proxy installation"
				return 1
			fi
			set -e
		fi
	    ;;
	*)
	    for dir in /etc/rc.d/init.d /etc/init.d /usr/sbin/init.d /sbin/init.d; do
	        [ -d $dir ] && MRCD=$dir
	    done

	    if [ "$INTERP" = "linux-ix86" -o "$INTERP" = "linux-s390" ]; then
	        MRCF="$MRCD/$proxy"
		if [ -d /var/lock/subsys ]; then
			touch /var/lock/subsys/$proxy
		fi
	    else
	        MRCF="$MRCD/$proxy.rc"
	    fi

	    if [ ! -d $MRCD ]; then
	        MRCF="$ETRC"
	    fi
	    if test -f $MRCF && cmp -s $MRCF $ETRC >$DEVNULL 2>&1; then
	        echo "$MRCF Tivoli $proxy daemon autostart file is already present."
	    else
	        rm -f $MRCF
	    	cp -p $ETRC $MRCF
		echo "Tivoli $proxy daemon master autostart file is $MRCF."
	    fi
	    for i in $CRCF; do
		if [ -f $i ]; then
		    rm -f $i;
		fi
		# the convention for hpux10 and sgi is to always
		# symlink startup scripts from init.d/script to
		# the corresponding rc?.d/script
		case "$INTERP" in
		    hpux10|mips-irix5|linux-ix86|linux-s390)
			ln -s $MRCF $i
			;;
		    *)
	        	ln $MRCF $i >$DEVNULL 2>&1
			if [ $? -ne 0 ]; then
			    ln -s $MRCF $i
			fi
			;;
		esac
	    done
            ;;
    esac

    return 0
}


work_dir=`pwd`

interp=`print_interp`

if [ X${interp} != Xaix4-r1 ] && [ X${interp} != Xsolaris2 ] && [ X${interp} != Xlinux-ix86 ] ; then
	echo "Platform $interp is not supported."
	exit 1
fi

ok=0
while [ $ok = 0 ] ; do
	echo $dashn "To install the Tivoli Event Sink, you must accept the agreement written in the License file."
	echo $dashn "If you accept the agreement, enter Y.  [Y/N]: "
	read ans

	if [ X${ans} = Xy ] || [ X${ans} = XY ] ||
         [ X${ans} = Xyes ] || [ X${ans} = XYES ] ; then
		ok=1
	fi

	if [ X${ans} = Xn ] || [ X${ans} = XN ] ||
         [ X${ans} = Xno ] || [ X${ans} = XNO ] ; then
		exit 1
	fi
done


ok=0
default=/usr/eventsink
while [ $ok = 0 ] ; do
	echo $dashn "Installation directory [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
	else
		try_path=${default}
	fi

	check_path ${try_path}
	if [ $? != 0 ] ; then
		echo $dashn "$try_path does not exist. Create it? [Y/N]: "
		check_response
		if [ $? = 0 ] ; then
			make_path $try_path
		fi
	fi

	check_path ${try_path}
	if [ $? = 0 ] ; then
		install_dir=${try_path}
		ok=1
	fi
done

ok=0
default=
while [ $ok = 0 ] ; do
	echo $dashn "LCF_DATDIR directory : "
	read ans
	if [ X${ans} != X ] ; then
		try_path=${ans}
		check_path ${try_path}
		if [ $? = 0 ] ; then
			mrt_dir=${try_path}/../../bin/${interp}/mrt
			check_path ${mrt_dir}
			if [ $? = 0 ] ; then
				lcf_datdir=${try_path}
				ok=1
			else
				echo $dashn "You must specify the fullpath to DAT directory of the endpoint "
			fi
		else
			echo $dashn "DAT directory specified does not exist "
		fi
	fi
done


ok=0
default=root
while [ $ok = 0 ] ; do
	echo $dashn "User account which proxy should be run as [default=$default]: "
	read ans
	if [ X${ans} = X ] ; then
		username=$default
	else	
		username=$ans
	fi
	ok=1
done

# set permissions on installed files
chown $username $install_dir
chmod 750 $install_dir
cp eventsink $install_dir/
chown $username $install_dir/*
chmod 700 $install_dir/*
mkdir $install_dir/C
chown $username $install_dir/C
chmod 750 $install_dir/C
cp evsk.cat $install_dir/C
chown $username $install_dir/C/*
chmod 700 $install_dir/C/*

ok=0
default=9444
while [ $ok = 0 ] ; do
	echo $dashn "Listening Port [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		[1-9][0-9][0-9][0-9][0-9])
			listening_port=$ans
			ok=1
			;;
		*)
			echo "Invalid port number -> " $ans
			;;
		esac
	else
		listening_port=$default
		ok=1
	fi
done

ok=0
default=50
while [ $ok = 0 ] ; do
	echo $dashn "Maximum Number of Events in Package [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9]*)
			max_num_evts=$ans
			ok=1
			;;
		*)
			echo "Invalid number -> " $ans
			;;
		esac
	else
		max_num_evts="${default}"
		ok=1
	fi
done

ok=0
default=40000
while [ $ok = 0 ] ; do
	echo $dashn "Maximum Buffer Size [default=$default]: "
	read ans
	if [ X${ans} != X ] ; then
		case $ans in
		[1-9][0-9]*)
			max_buf_sz=$ans
			ok=1
			;;
		*)
			echo "Invalid size -> " $ans
			;;
		esac
	else
		max_buf_sz="${default}"
		ok=1
	fi
done

cat >$install_dir/eventsink.cfg <<EOF
[SENDING]
lcf-datdir=$lcf_datdir
max-size-buffer=$max_buf_sz
max-num-events-to-send=$max_num_evts
delay-time=1
caching-timeout=30
[RECEPTION]
port=$listening_port
max-sessions=100
tcpip-timeout=120
max-ram-cache=1024
[EIF]
BufEvtMaxSize=64
BufEvtPath=$install_dir/eventsink.cache
[LOG]
log-file=$install_dir/eventsink.log
debug-level=3
file-size=1
EOF

nfile=unlimited
ulimit_solaris2="ulimit -n $nfile"

if [ X${interp} = Xsolaris2 ] ; then
    ulimit_cmd=${ulimit_solaris2}
fi

logfile_path=$install_dir/eventsink.log

# create the script to run the proxy
cat >$install_dir/eventsink.sh <<EOF
#!/bin/sh
INTERP=${interp}
export INTERP
INSTALL_DIR_PATH=$install_dir
export INSTALL_DIR_PATH
LCFROOT="$lcf_datdir/../.."
export LCFROOT
LCF_DATDIR="$lcf_datdir"
export LCF_DATDIR
LCF_BINDIR="\$LCFROOT/bin/\$INTERP/mrt"
export LCF_BINDIR
LCF_LIBDIR="\$LCFROOT/lib/\$INTERP"
export LCF_LIBDIR
LCF_CATDIR="\$LCFROOT/generic/msg_cat"
export LCF_CATDIR
LCF_TOOLSDIR="\$LCFROOT/bin/\$INTERP/tools"
export LCF_TOOLSDIR
PATH=/bin:/usr/bin:/usr/ucb:/usr/sbin:\$LCF_BINDIR:\$LCF_DATDIR:\$PATH
export PATH
NLSPATH=\$INSTALL_DIR_PATH/%L/%N.cat:\$INSTALL_DIR_PATH/%l/%N.cat:\$INSTALL_DIR_PATH/C/%N.cat:\$LCF_CATDIR/%L/%N.cat:\$LCF_CATDIR/%l/%N.cat:\$LCF_CATDIR/C/%N.cat\${NLSPATH:+":\$NLSPATH"}
export NLSPATH
TISDIR="\$LCF_DATDIR"
export TISDIR

# AIX
LIBPATH=\$LCF_LIBDIR:/usr/lib\${LIBPATH:+":\$LIBPATH"}
export LIBPATH
# Solaris, Linux
LD_LIBRARY_PATH=\$LCF_LIBDIR:/usr/openwin/lib:/usr/lib:/usr/ucblib\${LD_LIBRARY_PATH:+":\$LD_LIBRARY_PATH"}
export LD_LIBRARY_PATH
${ulimit_cmd}

[ -d "\$INSTALL_DIR_PATH" ] && cd \$INSTALL_DIR_PATH

case "\$1" in
	
	stop_msg)
		shift
		echo "Stopping the eventsink daemon"
		;;
	stop)
		shift
		if [ -f \$INSTALL_DIR_PATH/eventsink.pid ] ; then
			pid=\`sed -n 1p \$INSTALL_DIR_PATH/eventsink.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep eventsin | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						rm -f \$INSTALL_DIR_PATH/eventsink.pid
						kill -15 \$pid
						exit \$?
					fi
				done
				echo "The process id in \$INSTALL_DIR_PATH/eventsink.pid is not an eventsink process id."
				echo "Unable to get process id of the eventsink daemon (pid)."
			else
				echo "Unable to get process id of the eventsink daemon (pid)."
				exit 1
			fi
		else
			echo "Unable to get process id of the eventsink daemon (eventsink.pid)."
			exit 1
		fi
		;;
	start_msg)
		shift
		echo "Starting the eventsink daemon"
		;;
	start|*)
		if [ "\$1" = "start" ] ; then
			shift
		fi

		if [ -f \$INSTALL_DIR_PATH/eventsink.pid ] ; then
			pid=\`sed -n 1p \$INSTALL_DIR_PATH/eventsink.pid\`
			if [ -n "\$pid" ] ; then
				
				pidlist=\`ps -el | grep eventsin | grep -v grep | awk '{print \$4}'\`
				
				for temppid in \$pidlist
				do
					if [ \$temppid -eq \$pid ] ; then
						echo "eventsink for this environment already running under pid \$pid"
						exit 1
					fi
				done
			fi
		fi

		su - $username -c "cd \${INSTALL_DIR_PATH}; INTERP=\$INTERP LCFROOT=\$LCFROOT LCF_DATDIT=\$LCF_DATDIR LCF_BINDIR=\$LCF_BINDIR LCF_LIBDIR=\$LCF_LIBDIR LCF_CATDIR=\$LCF_CATDIR LCF_TOOLSDIR=\$LCF_TOOLSDIR PATH=\$PATH NLSPATH=\$NLSPATH TISDIR=\$TISDIR LIBPATH=\$LIBPATH LD_LIBRARY_PATH=\$LD_LIBRARY_PATH \$INSTALL_DIR_PATH/eventsink \$INSTALL_DIR_PATH" &

		;;
esac
EOF



# set permissions on the script
chown $username $install_dir/eventsink.sh
chmod 500 $install_dir/eventsink.sh

auto_start

ok=0
default="Y"
while [ $ok = 0 ] ; do
	echo $dashn "Start the event sink? [Y/N] [default=$default]: "
	check_response $default
	if [ $? = 0 ] ; then
		nohup ${install_dir}/eventsink.sh start 1>/dev/null 2>&1 &
	else
		echo "Event sink not started"
	fi
	ok=1
done
